// (C) Copyright 2003: Reece H. Dunn

// Produce the same formatting as the examples created by John Torjo for his
// list outputter library/tutorial

#include <iostream>  // std::cout
#include <list>      // std::list

#include <boost/io/formatlist.hpp>

template< class FormatType >
class position_output
{
   public:
      // needed for deduction of the nested format type
      typedef FormatType format_type;
   private:
      long pos;
   public:
      template< typename T, class OutputStream >
      OutputStream & operator()( OutputStream & os, const T & value ) const
      {
         return( os << '[' << const_cast< long & >( pos )++ << "] " << value );
      }
   public:
      position_output(): pos( 0 ){}
};

int main()
{
   // names - the data to be formatted

   std::list< std::string >            names;
   names.push_back( "John" );
   names.push_back( "James" );
   names.push_back( "Corina" );

   // [1]: John, James, Corina

   std::cout << "[1]: "
             << boost::io::formatlist( names ).format( "", "" )
             << '\n';

   // [2]: {John}, {James}, {Corina}

   std::cout << "[2]: "
             << boost::io::formatlistout( names, boost::io::wrappedfmt().format( "{", "}" ))
                .format( "", "" )
             << '\n';

   std::cout << "[2]: "
             << boost::io::formatlist( names )
                .format( "{", "}", "}, {" )
             << '\n';

   // [3]: {[0] John}, {[1] James}, {[2] Corina}

   std::cout << "[3]: "
             << boost::io::formatlistout
                (
                   names,
                   boost::io::wrappedfmtout( position_output< char * >()).format( "{", "}" )
                )
                .format( "", "" )
             << '\n';

   // [4]: [0] John, [1] James, [2] Corina

   std::cout << "[4]: "
             << boost::io::formatlistout( names, position_output< char * >())
                .format( "", "" )
             << '\n' << '\n';

   /*
      [5]:
      [John]
      [James]
      [Corina]
   */

   std::cout << "[5]:"
             << boost::io::formatlistout( names, boost::io::wrappedfmt().format( "[", "]" ))
                .format( "\n", "", "\n" )
             << '\n';

   /*
      [6]:
      [0] John
      [1] James
      [2] Corina
   */

   std::cout << "[6]:"
             << boost::io::formatlistout( names, position_output< char * >())
                .format( "\n", "", "\n" )
             << '\n' << '\n';

   /*
      [7]:
      '[0] John'
      '[1] James'
      '[2] Corina'
   */

   std::cout << "[7]:"
             << boost::io::formatlistout( names, position_output< char * >())
                .format( "\n'", "'", "'\n'" )
             << '\n' << '\n';

   return( 0 );
}
